require 'singleton'
require 'json'

module Nile

    class Router
        include Singleton
        attr_accessor :routes

        def initialize
            @routes = []
        end

        def add_route(verb, url, block)
            @routes << {:verb => verb, :url => url, :cb => block}
        end

        def find_route verb, url
            @routes.each do |route|
                if route[:url] == url && (route[:verb] == verb || verb == "HEAD")
                    return route
                end
            end
            return {:cb => Proc.new { [404, {}, ["Not found"]] } }
        end
    end

    class HttpHandler
        def handle req, sock
            start = Time.new
            r = Router.instance

            querystring = req[:url].split("?")
            params = {}
            if querystring[1].respond_to? :split
                querystring[1].split("&").each do |param|
                    param =~ /(\w+)=*(.*)/
                    params[$1] = $2
                end
            end

            route = r.find_route(req[:verb], querystring[0])
            begin
                response = route[:cb].call({:params => params, :req => req})
            rescue Exception => e
                response = [500, {}, ['Internal Server Error: ' + e.message]]
            end

            response[1]['Server'] = "Nile #{Nile::VERSION}"
            response[1]['Content-Length'] = response[2].join("\n").length

            return response
        end
    end

    class Core
        attr_accessor :router

        def initialize args={}
            args[:port] ||= 7125
            args[:bind] ||= '127.0.0.1'
            @http = Nile::HttpServer.new args[:bind], args[:port], HttpHandler.new
            @router = Router.instance
        end

        def get url, &block
            @router.add_route "GET", url, block
        end

        def post url, &block
            @router.add_route "POST", url, block
        end

        def put url, &block
            @router.add_route "PUT", url, block
        end

        def delete url, &block
            @router.add_route "DELETE", url, block
        end

        def run
            @http.serve
        end

    end


end
